local iconGold = GOLD_AMOUNT_TEXTURE
local iconSilver = SILVER_AMOUNT_TEXTURE
local iconCopper = COPPER_AMOUNT_TEXTURE

local AuctionsToGo = LibStub( "AceAddon-3.0" ):NewAddon( "Broker_AuctionsToGo", "AceEvent-3.0" )
local broker = LibStub("LibDataBroker-1.1", true)
local LibQTip = LibStub('LibQTip-1.0')
-- local L = LibStub("AceLocale-3.0"):GetLocale("Broker_SysMon")

AuctionsToGo.frame = CreateFrame("Frame")
local tooltip

local myauctions = {}
local ldb_text_string = "Current Auctions (me/realm): %d/%d"
local texture_format = "\124T%s:%d:%d:%d:%d\124t%s"
local auctime = { "short", "medium", "long", "very long" }

AuctionsToGo.ldb = broker:NewDataObject("Broker_AuctionsToGo", {
	suffix = "auctions",
	label = "AuctionsToGo",
	type = "data source",
	OnEnter = function(self)
		local db = AuctionsToGo.db.factionrealm

		tooltip = LibQTip:Acquire( "Broker_AuctionsToGo", 5, "LEFT", "RIGHT", "RIGHT", "RIGHT", "RIGHT")
		self.tooltip = tooltip
		tooltip:Clear()
		tooltip:AddLine("Auctions To Go")

		for charname,char in pairs(db) do
			-- DEFAULT_CHAT_FRAME:AddMessage("Broker: AuctionsToGo " .. charname)
			if (next(char.auctions) ~= nil) then
				tooltip:AddLine("Auctions  - " .. charname)
			end

			if (# char.auctions > 0) then
				tooltip:AddHeader("Item", "Current Bid", "Minumum bid", "Buy Out", "Run Time")
				for i,v in ipairs(char.auctions) do
					local minbidstringstring
					if (v.saleStatus == 1) then
						minbidstringstring = "sold"
					else
						minbidstringstring = AuctionsToGo:CreateMoneyString(v.minBid)
					end
					tooltip:AddLine(string.format(texture_format,v.texture, 24,24,1,0, v.link .. "x" .. v.count),  AuctionsToGo:CreateMoneyString(v.bidAmount), minbidstringstring, AuctionsToGo:CreateMoneyString(v.buyoutPrice), auctime[v.timeLeft])
				end
				if (char.buyoutSum) then
					tooltip:AddLine("Totals - " .. charname," "," ",AuctionsToGo:CreateMoneyString(char.buyoutSum))
				end
				tooltip:AddLine(" ")
			end
		end
		if (AuctionsToGo.db.realm.buyoutSum) then
			tooltip:AddLine(" ")
			tooltip:AddLine("Totals"," "," ", AuctionsToGo:CreateMoneyString(AuctionsToGo.db.realm.buyoutSum))
		end

		tooltip:EnableMouse()
		tooltip:SmartAnchorTo(self)
		tooltip:Show()
	end,
	OnLeave = function(self)
		LibQTip:Release(self.tooltip)
		self.tooltip = nil
	end,

})

function AuctionsToGo:CreateMoneyString(money)
	local moneystring = {}

	if (money == 0) then
		return 0
	end

	local copper = money % 100
	money = (money - copper) / 100
	local silver = money % 100
	local gold = floor(money / 100)

	if (gold > 0) then
		moneystring[# moneystring +1] = string.format(iconGold, gold, 16, 16)
		moneystring[# moneystring +1] = " "
	end

	if (gold + silver > 0) then
		moneystring[# moneystring +1] = string.format(iconGold, silver, 16, 16)
		moneystring[# moneystring +1] = " "
	end

	if (gold + silver + copper > 0) then
		moneystring[# moneystring +1] = string.format(iconGold, copper, 16, 16)
		moneystring[# moneystring +1] = " "
	end

	return table.concat(moneystring)
end

function AuctionsToGo:OnEnable()
	-- Called when the addon is loaded
	self.db = LibStub( "AceDB-3.0" ):New( "BATG_DB", nil, "Default" )
	if ( not self.db.factionrealm) then
		self.db.factionrealm = {}
	end

	if (not self.db.realm.auctioncount) then
		self.db.realm.auctioncount = 0
	end

	if (not self.db.realm.buyoutSum) then
		self.db.realm.buyoutSum = 0
	end

	AuctionsToGo.me = UnitName( "player" )
	AuctionsToGo.realm = GetRealmName()

	-- do we have any data yet for this toon? If not, init its table
	if not ( self.db.factionrealm[AuctionsToGo.me] ) then
		self.db.factionrealm[AuctionsToGo.me] = {
			auctions = {}
		}
	end

end

function AuctionsToGo:AUCTION_OWNED_LIST_UPDATE(eventname, ...)
	local dbfr = self.db.factionrealm[AuctionsToGo.me]
	local dbr = self.db.realm
	DEFAULT_CHAT_FRAME:AddMessage("Broker: AuctionsToGo dbrcount: " .. dbr.auctioncount)
	if ( dbfr.auctioncount) then
		if (dbr.auctioncount >= dbfr.auctioncount ) then
			dbr.auctioncount = dbr.auctioncount - dbfr.auctioncount
			DEFAULT_CHAT_FRAME:AddMessage("Broker: AuctionsToGo auctioncount substracting? " .. dbr.auctioncount)
		end
	end

	if (dbfr.buyoutSum and dbr.buyoutSum >= dbfr.buyoutSum ) then
		dbr.buyoutSum = dbr.buyoutSum - dbfr.buyoutSum
		DEFAULT_CHAT_FRAME:AddMessage("Broker: AuctionsToGo buyoutSum substracting? " .. dbr.buyoutSum)
	end
	dbfr.buyoutSum = 0


	_, dbfr.auctioncount = GetNumAuctionItems("owner")
	dbr.auctioncount = dbr.auctioncount + dbfr.auctioncount

	dbfr.auctions = {}

	AuctionsToGo.ldb.text = string.format(ldb_text_string, dbfr.auctioncount, dbr.auctioncount)

	for i=1,dbfr.auctioncount do
		local name, texture, count, quality, canUse, level, minBid, minIncrement, buyoutPrice, bidAmount, highBidder, owner, saleStatus = GetAuctionItemInfo("owner", i);
		local link = GetAuctionItemLink("owner", i);
		local timeLeft = GetAuctionItemTimeLeft("owner", i);
		dbfr.auctions[i] = {
			name = name,
			texture = texture,
			count = count,
			quality = quality,
			canUse = canUse,
			level = level,
			minBid = minBid,
			minIncrement = minIncrement,
			buyoutPrice = buyoutPrice,
			bidAmount = bidAmount,
			highBidder = highBidder,
			owner = owner,
			saleStatus = saleStatus,
			link = link,
			timeLeft = timeLeft,
		}
		dbfr.buyoutSum = dbfr.buyoutSum + buyoutPrice
		dbfr.buyoutSum = dbfr.buyoutSum + buyoutPrice
	end
	dbr.buyoutSum = dbr.buyoutSum + dbfr.buyoutSum
end

local total = 0
AuctionsToGo.frame:SetScript("OnUpdate", function(self, elapsed)
	total = total + elapsed
	if total < 1 then return end
	total = 0

	local dbr_count, dbfr_count = 0, 0
	if (AuctionsToGo.db.factionrealm[AuctionsToGo.me].auctioncount) then
		dbfr_count = AuctionsToGo.db.factionrealm[AuctionsToGo.me].auctioncount
	end
	if (AuctionsToGo.db.realm.auctioncount) then
		dfr_count = AuctionsToGo.db.realm.auctioncount
	end

	AuctionsToGo.ldb.text = string.format(ldb_text_string, dbfr_count, dfr_count)
end)

AuctionsToGo:RegisterEvent("AUCTION_OWNED_LIST_UPDATE")
